==> dxde_1.40.6-1.tar.gz: BUILD/src/print/documentprint.cpp <==
/***************************************************************
 * Name:      documentprint.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/log.h>
#include <wx/filename.h>
#include <wx/mimetype.h>
#include "wx/pdfdc.h"
#include "wx/pdffontmanager.h"
#include "documentprint.h"
#include "model/dbmodel.h"
#include "config.h"

DocumentPrint::DocumentPrint(wxWindow *parent, const wxString &title)
    : m_document(nullptr), m_headerLeftText(wxEmptyString), m_headerCenterText(wxEmptyString),
      m_headerRightText(wxEmptyString), m_headerLine(false), m_marginLeft(20-PAGE_OFFSET), m_marginRight(20+PAGE_OFFSET), m_marginTop(20-PAGE_OFFSET),
      m_marginBottom(20), m_pageWidth(210), m_pageHeight(297), m_parent(parent), m_title(title)
{
}

void DocumentPrint::Preview(bool print)
{
    DocumentPrintout *print1 = new DocumentPrintout(printTitle());
    print1->setDocument(m_document);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_document->pages());
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        DocumentPrintout *print2 = new DocumentPrintout(printTitle());
        print2->setDocument(m_document);
        print2->setHeaderLeftText(m_headerLeftText);
        print2->setHeaderCenterText(m_headerCenterText);
        print2->setHeaderRightText(m_headerRightText);
        print2->setHeaderLine(m_headerLine);
        print2->setMarginLeft(m_marginLeft);
        print2->setMarginRight(m_marginRight);
        print2->setMarginTop(m_marginTop);
        print2->setMarginBottom(m_marginBottom);
        print2->setPages(m_document->pages());
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(60);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(600,650));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void DocumentPrint::Print()
{
    DocumentPrintout *print = new DocumentPrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    print->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

wxString DocumentPrint::CreatePDF(const wxString &filename, const wxString &name, bool select)
{
    registerFontsForPdf();
    wxFileName fileName;
    if(select)
    {
        wxFileDialog dialog(m_parent, wxString::FromUTF8("Uložit fakturu jako"), wxGetHomeDir(), filename+wxT(".pdf"), wxT("PDF soubor (*.pdf)|*pdf"), wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
        if(dialog.ShowModal() == wxID_OK)
        {
            fileName.SetPath(dialog.GetDirectory());
            fileName.SetFullName(dialog.GetFilename());
        }
        else
        {
            return wxEmptyString;
        }
    }
    else
    {
        fileName.SetPath(wxStandardPaths::Get().GetUserDataDir());
        fileName.SetFullName(filename+wxT(".pdf"));
    }
    DocumentPrintout *print = new DocumentPrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintData data = print->printData();
    data.SetFilename(fileName.GetFullPath());
    wxPdfDC dc(data);
    dc.SetResolution(600);
    print->SetDC(&dc);
    print->OnPreparePrinting();
    print->OnBeginPrinting();
    int minPageNum = 1;
    int maxPageNum = m_document->pages();
    if(print->OnBeginDocument(minPageNum, maxPageNum))
    {
        dc.StartDoc("Start PDF...");
        wxPdfDocument *doc = dc.GetPdfDocument();
        if(!name.IsEmpty()) doc->SetTitle(name);
        doc->SetAuthor(idbmodel->firma().nazev());
        doc->SetCreator(wxString(FULLNAME));
        for(int pn = minPageNum; pn <= maxPageNum && print->HasPage(pn); pn++)
        {
            dc.StartPage();
            bool cont = print->OnPrintPage(pn);
            dc.EndPage();
            if(!cont) break;
        }
        print->OnEndDocument();
    }
    print->OnEndPrinting();
    return fileName.GetFullPath();
}

Document *DocumentPrint::document() const
{
    return m_document;
}

void DocumentPrint::setDocument(Document *document)
{
    m_document = document;
}

wxString DocumentPrint::headerLeftText() const
{
    return m_headerLeftText;
}

void DocumentPrint::setHeaderLeftText(const wxString &headerLeftText)
{
    m_headerLeftText = headerLeftText;
}

wxString DocumentPrint::headerCenterText() const
{
    return m_headerCenterText;
}

void DocumentPrint::setHeaderCenterText(const wxString &headerCenterText)
{
    m_headerCenterText = headerCenterText;
}

wxString DocumentPrint::headerRightText() const
{
    return m_headerRightText;
}

void DocumentPrint::setHeaderRightText(const wxString &headerRightText)
{
    m_headerRightText = headerRightText;
}

bool DocumentPrint::headerLine() const
{
    return m_headerLine;
}

void DocumentPrint::setHeaderLine(bool headerLine)
{
    m_headerLine = headerLine;
}

int DocumentPrint::marginLeft() const
{
    return m_marginLeft;
}

void DocumentPrint::setMarginLeft(int marginLeft)
{
    m_marginLeft = marginLeft;
}

int DocumentPrint::marginRight() const
{
    return m_marginRight;
}

void DocumentPrint::setMarginRight(int marginRight)
{
    m_marginRight = marginRight;
}

int DocumentPrint::marginTop() const
{
    return m_marginTop;
}

void DocumentPrint::setMarginTop(int marginTop)
{
    m_marginTop = marginTop;
}

int DocumentPrint::marginBottom() const
{
    return m_marginBottom;
}

void DocumentPrint::setMarginBottom(int marginBottom)
{
    m_marginBottom = marginBottom;
}

int DocumentPrint::pageWidth() const
{
    return m_pageWidth;
}

void DocumentPrint::setPageWidth(int pageWidth)
{
    m_pageWidth = pageWidth;
}

int DocumentPrint::pageHeight() const
{
    return m_pageHeight;
}

void DocumentPrint::setPageHeight(int pageHeight)
{
    m_pageHeight = pageHeight;
}

wxString DocumentPrint::printTitle() const
{
    if(m_title.StartsWith(" "))
        return m_title;
    return wxString(" ")+m_title;
}

void DocumentPrint::registerFontsForPdf()
{
    if(m_document)
    {
        wxPdfFontManager* fontManager = wxPdfFontManager::GetFontManager();
        dxElementsArray elements = m_document->elements();
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            if(elements[i]->elementType() == TEXT || elements[i]->elementType() == TEXTPARAGRAPH
                    || elements[i]->elementType() == TABLEROW)
            {
                fontManager->RegisterFont(elements[i]->style().font());
            }
        }
    }
}

DocumentPrintout::DocumentPrintout(const wxString &title)
    : dxPrint(title), m_document(nullptr)
{
}

bool DocumentPrintout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    int ydist = 0, xdist = 0;
    if(dc->IsKindOf(CLASSINFO(wxPdfDC)))
    {
        ydist = PAGE_OFFSET;
        xdist = PAGE_OFFSET;
    }
    drawHeader(dc, m_mmToLogical);
    if(m_document)
    {
        dxElementsArray elements = m_document->elementsByPage(page);
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            elements[i]->DrawToDC(dc, m_mmToLogical, ydist, xdist);
        }
    }
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void DocumentPrintout::setDocument(Document *document)
{
    if(document)
        m_document = document;
}

==> dxde_1.40.12-1.tar.gz: BUILD/src/print/documentprint.cpp <==
/***************************************************************
 * Name:      documentprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/log.h>
#include <wx/filename.h>
#include <wx/mimetype.h>
#include "wx/pdfdc.h"
#include "wx/pdffontmanager.h"
#include "documentprint.h"
#include "model/dbmodel.h"
#include "config.h"

DocumentPrint::DocumentPrint(wxWindow *parent, const wxString &title)
    : m_document(nullptr), m_headerLeftText(wxEmptyString), m_headerCenterText(wxEmptyString),
      m_headerRightText(wxEmptyString), m_headerLine(false), m_marginLeft(20-PAGE_OFFSET), m_marginRight(20+PAGE_OFFSET), m_marginTop(20-PAGE_OFFSET),
      m_marginBottom(20), m_pageWidth(210), m_pageHeight(297), m_parent(parent), m_title(title)
{
}

void DocumentPrint::Preview(bool print)
{
    DocumentPrintout *print1 = new DocumentPrintout(printTitle());
    print1->setDocument(m_document);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_document->pages());
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        DocumentPrintout *print2 = new DocumentPrintout(printTitle());
        print2->setDocument(m_document);
        print2->setHeaderLeftText(m_headerLeftText);
        print2->setHeaderCenterText(m_headerCenterText);
        print2->setHeaderRightText(m_headerRightText);
        print2->setHeaderLine(m_headerLine);
        print2->setMarginLeft(m_marginLeft);
        print2->setMarginRight(m_marginRight);
        print2->setMarginTop(m_marginTop);
        print2->setMarginBottom(m_marginBottom);
        print2->setPages(m_document->pages());
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTH,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void DocumentPrint::Print()
{
    DocumentPrintout *print = new DocumentPrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintDialogData data(print->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    print->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    wxPrinter printer(&data);
    printer.Print(m_parent, print, true);
}

wxString DocumentPrint::CreatePDF(const wxString &filename, const wxString &name, bool select)
{
    registerFontsForPdf();
    wxFileName fileName;
    if(select)
    {
        wxFileDialog dialog(m_parent, wxString::FromUTF8("Uložit fakturu jako"), wxGetHomeDir(), filename+wxT(".pdf"), wxT("PDF soubor (*.pdf)|*pdf"), wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
        if(dialog.ShowModal() == wxID_OK)
        {
            fileName.SetPath(dialog.GetDirectory());
            fileName.SetFullName(dialog.GetFilename());
        }
        else
        {
            return wxEmptyString;
        }
    }
    else
    {
        fileName.SetPath(wxStandardPaths::Get().GetUserDataDir());
        fileName.SetFullName(filename+wxT(".pdf"));
    }
    DocumentPrintout *print = new DocumentPrintout(printTitle());
    print->setDocument(m_document);
    print->setHeaderLeftText(m_headerLeftText);
    print->setHeaderCenterText(m_headerCenterText);
    print->setHeaderRightText(m_headerRightText);
    print->setHeaderLine(m_headerLine);
    print->setMarginLeft(m_marginLeft);
    print->setMarginRight(m_marginRight);
    print->setMarginTop(m_marginTop);
    print->setMarginBottom(m_marginBottom);
    print->setPages(m_document->pages());
    print->performPageSetup();
    wxPrintData data = print->printData();
    data.SetFilename(fileName.GetFullPath());
    wxPdfDC dc(data);
    dc.SetResolution(600);
    print->SetDC(&dc);
    print->OnPreparePrinting();
    print->OnBeginPrinting();
    int minPageNum = 1;
    int maxPageNum = m_document->pages();
    if(print->OnBeginDocument(minPageNum, maxPageNum))
    {
        dc.StartDoc("Start PDF...");
        wxPdfDocument *doc = dc.GetPdfDocument();
        if(!name.IsEmpty()) doc->SetTitle(name);
        doc->SetAuthor(idbmodel->firma().nazev());
        doc->SetCreator(wxString(FULLNAME));
        for(int pn = minPageNum; pn <= maxPageNum && print->HasPage(pn); pn++)
        {
            dc.StartPage();
            bool cont = print->OnPrintPage(pn);
            dc.EndPage();
            if(!cont) break;
        }
        print->OnEndDocument();
    }
    print->OnEndPrinting();
    return fileName.GetFullPath();
}

Document *DocumentPrint::document() const
{
    return m_document;
}

void DocumentPrint::setDocument(Document *document)
{
    m_document = document;
}

wxString DocumentPrint::headerLeftText() const
{
    return m_headerLeftText;
}

void DocumentPrint::setHeaderLeftText(const wxString &headerLeftText)
{
    m_headerLeftText = headerLeftText;
}

wxString DocumentPrint::headerCenterText() const
{
    return m_headerCenterText;
}

void DocumentPrint::setHeaderCenterText(const wxString &headerCenterText)
{
    m_headerCenterText = headerCenterText;
}

wxString DocumentPrint::headerRightText() const
{
    return m_headerRightText;
}

void DocumentPrint::setHeaderRightText(const wxString &headerRightText)
{
    m_headerRightText = headerRightText;
}

bool DocumentPrint::headerLine() const
{
    return m_headerLine;
}

void DocumentPrint::setHeaderLine(bool headerLine)
{
    m_headerLine = headerLine;
}

int DocumentPrint::marginLeft() const
{
    return m_marginLeft;
}

void DocumentPrint::setMarginLeft(int marginLeft)
{
    m_marginLeft = marginLeft;
}

int DocumentPrint::marginRight() const
{
    return m_marginRight;
}

void DocumentPrint::setMarginRight(int marginRight)
{
    m_marginRight = marginRight;
}

int DocumentPrint::marginTop() const
{
    return m_marginTop;
}

void DocumentPrint::setMarginTop(int marginTop)
{
    m_marginTop = marginTop;
}

int DocumentPrint::marginBottom() const
{
    return m_marginBottom;
}

void DocumentPrint::setMarginBottom(int marginBottom)
{
    m_marginBottom = marginBottom;
}

int DocumentPrint::pageWidth() const
{
    return m_pageWidth;
}

void DocumentPrint::setPageWidth(int pageWidth)
{
    m_pageWidth = pageWidth;
}

int DocumentPrint::pageHeight() const
{
    return m_pageHeight;
}

void DocumentPrint::setPageHeight(int pageHeight)
{
    m_pageHeight = pageHeight;
}

wxString DocumentPrint::printTitle() const
{
    if(m_title.StartsWith(" "))
        return m_title;
    return wxString(" ")+m_title;
}

void DocumentPrint::registerFontsForPdf()
{
    if(m_document)
    {
        wxPdfFontManager* fontManager = wxPdfFontManager::GetFontManager();
        dxElementsArray elements = m_document->elements();
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            if(elements[i]->elementType() == TEXT || elements[i]->elementType() == TEXTPARAGRAPH
                    || elements[i]->elementType() == TABLEROW)
            {
                fontManager->RegisterFont(elements[i]->style().font());
            }
        }
    }
}

DocumentPrintout::DocumentPrintout(const wxString &title)
    : dxPrint(title), m_document(nullptr)
{
}

bool DocumentPrintout::OnPrintPage(int page)
{
    wxDC* dc = GetDC();
    if (dc==nullptr || !dc->IsOk())
    {
        wxLogError(wxString::FromUTF8("Chyba při přípravě tisku, ukončuji tisk"));
        return false;
    }
    int ydist = 0, xdist = 0;
    if(dc->IsKindOf(CLASSINFO(wxPdfDC)))
    {
        ydist = PAGE_OFFSET;
        xdist = PAGE_OFFSET;
    }
    drawHeader(dc, m_mmToLogical);
    if(m_document)
    {
        dxElementsArray elements = m_document->elementsByPage(page);
        for(size_t i=0; i<elements.GetCount(); i++)
        {
            elements[i]->DrawToDC(dc, m_mmToLogical, ydist, xdist);
        }
    }
    drawFooter(page, dc, m_mmToLogical);
    return true;
}

void DocumentPrintout::setDocument(Document *document)
{
    if(document)
        m_document = document;
}
